/*
 *  Copyright (c) 2003-2004 Jesper K. Pedersen <blackie@kde.org>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 **/

#ifndef OPTIONS_H
#define OPTIONS_H
#include <qstringlist.h>
#include <qmap.h>
#include <qpixmap.h>
#include <qobject.h>
#include <qdom.h>
#include "membermap.h"
#include "imagesearchinfo.h"
class ImageConfig;

class Options :public QObject {
    Q_OBJECT

public:
    static Options* instance();
    static void setup( const QDomElement& config, const QDomElement& options, const QDomElement& configWindowSetup, const QDomElement& memberGroups, const QString& imageDirectory );

    void setThumbSize( int );
    int thumbSize() const;

    void setPreviewSize( int );
    int previewSize() const;

    void setMaxImages( int );
    int maxImages() const;

    void setViewerSize( int width, int height );
    QSize viewerSize() const;

    // -------------------------------------------------- Options
    void setOption( const QString& optionGroup,  const QStringList& value );
    void addOption( const QString& optionGroup,  const QString& value );
    void removeOption( const QString& optionGroup, const QString& value );
    QStringList optionValue( const QString& optionGroup ) const;
    QStringList optionFullValue( const QString& optionGroup ) const;
    void renameOption( const QString& optionGroup, const QString& oldValue, const QString& newValue );

    void setOptionImage( const QString& optionGroup, QString, const QImage& image );
    QImage optionImage( const QString& optionGroup,  QString, int size ) const;

    // -------------------------------------------------- Option Groups
    enum ViewSize { Small, Large };
    enum ViewType { ListView, IconView };
    enum ViewSortType { SortLastUse, SortAlpha };

    struct OptionGroupInfo
    {
        OptionGroupInfo() {}
        OptionGroupInfo( const QString& icon, ViewSize size, ViewType type, bool show = true, ViewSortType sortType = SortLastUse )
            : _icon(icon), _show(show), _size( size ), _type( type ), _sortType( sortType ) {}
        QString _icon;
        bool _show;
        ViewSize _size;
        ViewType _type;
	ViewSortType _sortType;
    };

    QStringList optionGroups() const;
    void addOptionGroup( const QString& name, const QString& icon, ViewSize size, ViewType type );
    void deleteOptionGroup( const QString& name );
    void renameOptionGroup( const QString& oldName, const QString& newName );

    QString textForOptionGroup( const QString& name ) const;

    QPixmap iconForOptionGroup( const QString& name, int size = 22 ) const;
    QString iconNameForOptionGroup( const QString& name ) const;
    void setIconForOptionGroup( const QString& name, const QString& icon );

    void setViewSize( const QString& optionGroup, ViewSize size );
    void setViewType( const QString& optionGroup, ViewType type );
    void setViewSortType( const QString& optionGroup, ViewSortType sortType );
    ViewSize viewSize( const QString& optionGroup ) const;
    ViewType viewType( const QString& optionGroup ) const;
    ViewSortType viewSortType( const QString& optionGroup ) const;

    // -------------------------------------------------- Member Groups
    const MemberMap& memberMap();
    void setMemberMap( const MemberMap& );

    // -------------------------------------------------- Options for the Viewer
    enum Position { Bottom = 0, Top, Left, Right, TopLeft, TopRight, BottomLeft, BottomRight };
    bool showInfoBox() const;
    bool showDrawings() const;
    bool showDescription() const;
    bool showDate() const;
    bool showOption( const QString& ) const;

    void setShowInfoBox(bool b);
    void setShowDrawings(bool b);
    void setShowDescription(bool b);
    void setShowDate(bool b);
    void setShowOption( const QString& optionGroup, bool b );

    Position infoBoxPosition() const;
    void setInfoBoxPosition( Position pos );

    // -------------------------------------------------- misc
    enum TimeStampTrust {
        Always = 0,
        Ask = 1,
        Never = 2
    };

    bool trustTimeStamps();
    void setTTimeStamps( TimeStampTrust );
    TimeStampTrust tTimeStamps() const;

    void setUseEXIFRotate( bool b );
    bool useEXIFRotate() const;

    void setAutoSave( int min );
    int autoSave() const;

    void save( QDomElement top );

    QString imageDirectory() const;

    QString HTMLBaseDir() const;
    void setHTMLBaseDir( const QString& dir );

    QString HTMLBaseURL() const;
    void setHTMLBaseURL( const QString& dir );

    void saveConfigWindowLayout( ImageConfig* );
    void loadConfigWindowLayout( ImageConfig* );

    void setCurrentScope( const ImageSearchInfo&, bool exclude );
    ImageSearchInfo currentScope() const;

    void setLocked( bool );
    bool isLocked() const;
    bool lockExcludes() const;

    void setPassword( const QString& passwd );
    QString password() const;

signals:
    void optionGroupsChanged();
    void changed();
    void renamedOption( const QString& optionGroup, const QString& oldName, const QString& newName );
    void deletedOption( const QString& optionGroup, const QString& name );
    void locked( bool lock, bool exclude );

private:
    Options( const QDomElement& config, const QDomElement& options, const QDomElement& configWindowSetup, const QDomElement& memberGroups, const QString& imageDirectory  );
    static Options* _instance;

    int _thumbSize,  _previewSize;
    TimeStampTrust _tTimeStamps;
    bool _useEXIFRotate;
    int _autoSave, _maxImages;
    bool _trustTimeStamps, _markNew, _hasAskedAboutTimeStamps, _ensureImageWindowsOnScreen;
    QMap<QString, QStringList> _options;
    QMap<QString,OptionGroupInfo> _optionGroups;
    QString _imageDirectory, _htmlBaseDir, _htmlBaseURL;

    Position _infoBoxPosition;
    bool _showInfoBox, _showDrawings, _showDescription, _showDate;
    QDomElement _configDock;

    QSize _viewerSize;

    MemberMap _members;
    ImageSearchInfo _currentScope;
    bool _locked, _exclude;
    QString _passwd;
};

#endif /* OPTIONS_H */

